@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-let header = "$Id: htmlCommon.ml,v 1.37 2002/03/19 10:51:38 maranget Exp $" 
+let header = "$Id: htmlCommon.ml,v 1.40 2003/03/07 17:25:33 maranget Exp $" 
 
 (* Output function for a strange html model :
      - Text elements can occur anywhere and are given as in latex
@@ -127,6 +127,13 @@ let check_block_closed opentag closetag =
     failclose "html" closetag opentag
 ;;
 
+let display_arg  verbose =
+  if verbose > 1 then
+    "BORDER=1 CELLSPACING=0 CELLPADDING=0"
+  else
+    "CELLSPACING=0 CELLPADDING=0"
+;;
+
 (* output globals *)
 type t_env = {here : bool ; env : text}
 
@@ -172,6 +179,9 @@ let pretty_top_styles stderr {top_pending = pending ; top_active = active} =
     pretty_mods pending
     pretty_tmods active
 
+let tbool = function
+  | true -> "+"
+  | false -> "-"
 
 let pretty_top stderr = function
   | Nothing x -> Printf.fprintf stderr "Nothing %a"  pretty_top_styles x
@@ -180,7 +190,7 @@ let pretty_top stderr = function
   | ActivateClosed _ -> Printf.fprintf stderr "ActivateClosed"
   | NotMe -> Printf.fprintf stderr "NotMe"
   | Insert (b,active) ->
-      Printf.fprintf stderr "Insert %b %a" b pretty_mods active
+      Printf.fprintf stderr "Insert %s %a" (tbool b) pretty_mods active
 
 type status = {
   mutable nostyle : bool ;
@@ -711,11 +721,16 @@ let forget_par () =
 
   
 let do_close_mod = function
-  Style m ->  
+| Style m ->
     if flags.in_math && !Parse_opts.mathml then 
       if m="mtext" then do_put ("</"^m^">")
       else do_put "</mstyle>"
     else do_put ("</"^m^">")
+| StyleAttr (t,_) -> 
+    if flags.in_math && !Parse_opts.mathml then 
+      ()
+    else
+      do_put ("</"^t^">")
 | (Color _ | Font _)  -> 
     if flags.in_math && !Parse_opts.mathml then 
       do_put "</mstyle>"
@@ -725,7 +740,7 @@ and do_open_mod e =
   if !verbose > 3 then
       prerr_endline ("do_open_mod: "^pretty_text e) ;
   match e with
-  Style m ->  
+  | Style m ->  
     if flags.in_math && !Parse_opts.mathml then 
       if m="mtext" then do_put ("<"^m^">")
       else do_put ("<mstyle style = \""^
@@ -737,6 +752,10 @@ and do_open_mod e =
 		   | _ -> m)^
 		   "\">")
     else do_put ("<"^m^">")
+  | StyleAttr (t,a) ->
+      if flags.in_math && !Parse_opts.mathml then ()
+      else
+        do_put ("<"^t^" "^a^">")
 | Font i  ->
     if flags.in_math && !Parse_opts.mathml then 
       do_put ("<mstyle style = \"font-size: "^string_of_int i^"\">")
@@ -823,8 +842,8 @@ let close_mods () = do_close_mods ()
 
 
 let is_style = function
-  Style _ -> true
-| _ -> false
+  |  Style _|StyleAttr (_,_) -> true
+  | _ -> false
 
 and is_font = function
   Font _ -> true
@@ -833,6 +852,7 @@ and is_font = function
 and is_color = function
   Color _ -> true
 | _ -> false
+
 ;;
 
 let do_open_these_mods do_open_mod pending =
@@ -1021,7 +1041,7 @@ let already_here = function
 | x ->
   first_same x
    (match x with
-     Style _ ->  is_style
+     Style _|StyleAttr (_,_) ->  is_style
    | Font _ -> is_font
    | Color _ -> is_color)
    !cur_out
@@ -1087,6 +1107,8 @@ let erase_mod_pred pred same_constr =
 
 let same_env = function
   | Style s1 -> (function | Style s2 -> s1 = s2 | _ -> false)
+  | StyleAttr (t1,a1) ->
+      (function | StyleAttr (t2,a2)-> t1 = t2 && a1=a2 | _ -> false)
   | Font i1 ->
       (function | Font i2 -> i1 = i2 | _ -> false)
   | Color s1 ->
@@ -1095,7 +1117,7 @@ let same_env = function
 and same_constr = function
   | Color _ -> is_color
   | Font _ -> is_font
-  | Style _ -> is_style
+  | Style _|StyleAttr (_,_) -> is_style
 
 let erase_mods ms =
   let rec erase_rec = function
@@ -1222,7 +1244,7 @@ let rec do_open_block insert s args = match s with
    | _ -> ()
    end
 | DISPLAY ->
-   do_open_block insert TABLE "" ;
+   do_open_block insert TABLE (display_arg !verbose) ;
    do_open_block None TR args
 | _  -> begin match insert with
   | Some (tag,iargs) ->
