@@ -9,13 +9,13 @@
 (*                                                                     *)
 (***********************************************************************)
 
-let header = "$Id: htmlMath.ml,v 1.14 2000/05/05 07:13:57 maranget Exp $" 
+let header = "$Id: htmlMath.ml,v 1.23 2003/03/07 17:25:33 maranget Exp $" 
 
 
 open Misc
 open Parse_opts
+open Element
 open HtmlCommon
-open Latexmacros
 open Stack
 
 
@@ -25,23 +25,21 @@ let delay_stack = Stack.create "delay_stack"
 (* delaying output .... *)
 
 let delay f =
-  if !verbose > 2 then
-    prerr_flags "=> delay" ;
+  if !verbose > 2 then prerr_flags "=> delay" ;
   push stacks.s_vsize flags.vsize ;
   flags.vsize <- 0;
   push delay_stack f ;
-  open_block "DELAY" "" ;
-  if !verbose > 2 then
-    prerr_flags "<= delay"
+  open_block DELAY "" ;
+  if !verbose > 2 then prerr_flags "<= delay"
 ;;
 
 let flush x =
   if !verbose > 2 then
     prerr_flags ("=> flush arg is ``"^string_of_int x^"''");
-  try_close_block "DELAY" ;
+  try_close_block DELAY ;
   let ps,_,pout = pop_out out_stack in
-  if ps <> "DELAY" then
-    raise (Misc.Fatal ("html: Flush attempt on: "^ps)) ;
+  if ps <> DELAY then
+    raise (Misc.Fatal ("html: Flush attempt on: "^string_of_block ps)) ;
   let mods = as_envs !cur_out.active !cur_out.pending in
   do_close_mods () ;
   let old_out = !cur_out in
@@ -78,16 +76,10 @@ let put_in_math s =
 (* DISPLAYS *)
 (*----------*)
 
-let open_center () =  open_block "DIV" "ALIGN=center"
-and close_center () = close_block "DIV"
+let open_center () =  open_block DIV "ALIGN=center"
+and close_center () = close_block DIV
 ;;
 
-let display_arg  verbose =
-  if verbose > 1 then
-    "BORDER=1 CELLSPACING=0 CELLPADDING=0"
-  else
-    "CELLSPACING=0 CELLPADDING=0"
-;;
 
 
 
@@ -97,15 +89,15 @@ let begin_item_display f is_freeze =
     Printf.fprintf stderr "begin_item_display: ncols=%d empty=%s" flags.ncols (sbool flags.empty) ;
     prerr_newline ()
   end ;
-  open_block "TD" "NOWRAP";
-  open_block "" "" ;
+  open_block TD "NOWRAP";
+  open_block INTERN "" ;
   if is_freeze then(* push out_stack (Freeze f) ;*)freeze f;
 
 
 and end_item_display () =
   let f,is_freeze = pop_freeze () in
-  let _ = close_flow_loc "" in
-  if close_flow_loc "TD" then
+  let _ = close_flow_loc INTERN in
+  if close_flow_loc TD then
     flags.ncols <- flags.ncols + 1;
   if !verbose > 2 then begin
     Printf.fprintf stderr "end_item_display: ncols=%d stck: " flags.ncols;
@@ -114,27 +106,31 @@ and end_item_display () =
   flags.vsize,f,is_freeze
 ;;
 
-let open_display () =
+let open_display_varg varg =
   if !verbose > 2 then begin
     Printf.fprintf stderr "open_display: "
   end ;
   try_open_display () ;
-  open_block "DISPLAY" (display_arg !verbose) ;
-  open_block "TD" "NOWRAP" ;
-  open_block "" "" ;
+  open_block DISPLAY varg ;
+  open_block TD "NOWRAP" ;
+  open_block INTERN "" ;
   if !verbose > 2 then begin
     pretty_cur !cur_out ;
     prerr_endline ""
   end     
 ;;
 
+let open_display () = open_display_varg "VALIGN=middle"
+
 let close_display () =
   if !verbose > 2 then begin
     prerr_flags "=> close_display"
   end ;
   if not (flush_freeze ()) then begin
-    close_flow "" ;
+    close_flow INTERN ;
     let n = flags.ncols in
+    if !verbose > 2 then
+      Printf.fprintf stderr "=> close_display, ncols=%d\n" n ;
     if (n = 0 && not flags.blank) then begin
       if !verbose > 2 then begin
         prerr_string "No Display n=0" ;
@@ -144,14 +140,14 @@ let close_display () =
       let active = !cur_out.active and pending = !cur_out.pending in
       do_close_mods () ;
       let ps,_,pout = pop_out out_stack in
-      if ps <> "TD" then
-        failclose ("close_display: "^ps^" closes TD") ;
+      if ps <> TD then
+        failclose "close_display" ps TD ;
       do_close_mods () ;
-      try_close_block "TD" ;
+      try_close_block TD ;
       let ps,_,ppout = pop_out out_stack in
-      if ps <> "DISPLAY" then
-        failclose ("close_display: "^ps^" closes DISPLAY") ;
-      try_close_block "DISPLAY" ;
+      if ps <> DISPLAY then
+        failclose "close_display" ps DISPLAY;
+      try_close_block DISPLAY ;
       let old_out = !cur_out in
       cur_out := ppout ;
       do_close_mods () ;
@@ -165,12 +161,12 @@ let close_display () =
         (Out.debug stderr !cur_out.out);
         prerr_endline "" ;
       end;
-      close_flow "FORGET" ;
+      close_flow FORGET ;
       let active = !cur_out.active and pending = !cur_out.pending in
       let ps,_,pout = pop_out out_stack in
-      if ps <> "DISPLAY" then
-        failclose ("close_display: "^ps^" closes DISPLAY") ;
-      try_close_block "DISPLAY" ;
+      if ps <> DISPLAY then
+        failclose "close_display" ps DISPLAY ;
+      try_close_block DISPLAY ;
       let old_out = !cur_out in
       cur_out := pout ;
       do_close_mods () ;
@@ -185,8 +181,8 @@ let close_display () =
         prerr_endline ""
       end;
       flags.empty <- flags.blank ;
-      close_flow "TD" ;
-      close_flow "DISPLAY"
+      close_flow TD ;
+      close_flow DISPLAY
     end ;
     try_close_display ()
   end ;
@@ -207,12 +203,12 @@ let do_item_display force =
     let active  = !cur_out.active
     and pending = !cur_out.pending in
     flags.ncols <- flags.ncols + 1 ;
-    let save = get_block "TD" "NOWRAP" in
+    let save = get_block TD "NOWRAP" in
     if !verbose > 2 then begin
       Out.debug stderr !cur_out.out ;
       prerr_endline "To be copied"
     end;
-    if close_flow_loc "TD" then flags.ncols <- flags.ncols + 1; 
+    if close_flow_loc TD then flags.ncols <- flags.ncols + 1; 
     if !verbose > 2 then begin
       Out.debug stderr !cur_out.out ;
       prerr_endline "Was copied"
@@ -226,21 +222,21 @@ let do_item_display force =
       Out.debug stderr !cur_out.out ;
       prerr_endline ("Some Item")
     end;
-    open_block "TD" "NOWRAP" ;
-    open_block "" ""
+    open_block TD "NOWRAP" ;
+    open_block INTERN ""
   end else begin
     if !verbose > 2 then begin
       Out.debug stderr !cur_out.out ;
       prerr_endline "No Item" ;
       prerr_endline ("flags: empty="^sbool flags.empty^" blank="^sbool flags.blank)
     end;
-    close_flow "" ;
+    close_flow INTERN ;
     if !verbose > 2 then begin
       Out.debug stderr !cur_out.out ;
       prerr_endline "No Item" ;
       prerr_endline ("flags: empty="^sbool flags.empty^" blank="^sbool flags.blank)
     end;
-    open_block "" ""
+    open_block INTERN ""
   end ;
   if is_freeze then push out_stack (Freeze f) ;
   if !verbose > 2 then begin
@@ -255,9 +251,9 @@ and force_item_display () = do_item_display true
 
 
 let erase_display () =
-  erase_block "" ;
-  erase_block "TD" ;
-  erase_block "DISPLAY" ;
+  erase_block INTERN ;
+  erase_block TD ;
+  erase_block DISPLAY ;
   try_close_display ()
 ;;
 
@@ -287,104 +283,85 @@ let open_vdisplay display =
     prerr_endline "open_vdisplay";
   if not display then
     raise (Misc.Fatal ("VDISPLAY in non-display mode"));
-  open_block "TABLE" (display_arg !verbose)
+  open_block TABLE (display_arg !verbose)
 
 and close_vdisplay () =
   if !verbose > 1 then
     prerr_endline "close_vdisplay";
-  close_block "TABLE"
+  close_block TABLE
 
 and open_vdisplay_row s =
   if !verbose > 1 then
     prerr_endline "open_vdisplay_row";
-  open_block "TR" "" ;
-  open_block "TD" s ;
+  open_block TR "" ;
+  open_block TD s ;
   open_display ()
 
 and close_vdisplay_row () =
   if !verbose > 1 then
     prerr_endline "close_vdisplay_row";
   close_display () ;
-  force_block "TD" "&nbsp;" ;
-  close_block "TR"
+  force_block TD "&nbsp;" ;
+  close_block TR
 ;;
 
 
 
 (* Sup/Sub stuff *)
 
-type ital = Ital | NoItal | Complex | Mixed
-;;
-
-let check_char = function
-  '{' | '}' | '$' | '^' | '_' | '\\' | '#' -> Complex
-| c ->
-   if ((c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z')) then
-     Ital
-   else NoItal
-;;
-
-exception Over
-;;
-
-let check_ital s =
-  let rec check_rec = function
-    -1 -> raise (Misc.Fatal "Empty string in check_rec")
-  | 0  -> check_char (String.get s 0)
-  | i  ->
-     let t = check_char (String.get s i)
-     and tt = check_rec (i-1) in
-     match t,tt with
-       Ital,Ital -> Ital
-     | NoItal,NoItal -> NoItal
-     | Ital,NoItal   -> Mixed
-     | NoItal,Ital   -> Mixed
-     | Complex,_ -> raise Over
-     | _,Complex -> raise Over
-     | _,Mixed   -> Mixed
-     | Mixed,_   -> Mixed in
-
-  match s with "" -> NoItal
-  | _ ->
-     try check_rec (String.length s-1) with Over -> Complex
-;;
-
-
-
 let get_script_font () =
   let n = get_fontsize () in
-  if n >= 3 then n-1 else n
+  if n >= 3 then Some (n-1) else None
 ;;
 
 let open_script_font () =
-  open_mod (Font (get_script_font ()))
+  if not !pedantic then
+    match get_script_font () with
+    | Some m -> open_mod (Font m)
+    | _ -> ()
 ;;
 
 
-let complex s = match check_ital s with
-| Complex -> true
-| _       -> false
-;;
-
-let put_sup_sub tag scanner = function
-  "" -> ()
-| s  ->
-    open_group tag ;
-    if not !pedantic then open_script_font () ;
-    scanner s;
-    close_group ()
+let put_sup_sub display scanner (arg : string Lexstate.arg) =
+  if display then open_display () else open_block INTERN "" ;
+  open_script_font () ;
+  scanner arg ;
+  if display then close_display () else close_block INTERN ;
 ;;
 
+let reput_sup_sub tag = function
+  | "" -> ()
+  | s  ->
+      open_block INTERN "" ;
+      clearstyle () ;
+      if not  (flags.in_pre && !pedantic) then begin
+        put_char '<' ;
+        put tag ;
+        put_char '>'
+      end ;
+      put s ;
+      if not  (flags.in_pre && !pedantic) then begin
+        put "</" ;
+        put tag ;
+        put_char '>'
+      end ;
+      close_block INTERN
 
 
 let standard_sup_sub scanner what sup sub display =
-  if display && (complex sup || complex sub) then begin
+  let sup,fsup =
+    hidden_to_string (fun () -> put_sup_sub display scanner sup)
+  in
+  let sub,fsub =
+    hidden_to_string (fun () -> put_sup_sub display scanner sub) in
+
+  if display && (fsub.table_inside || fsup.table_inside) then begin
     force_item_display () ;
     open_vdisplay display ;
     if sup <> "" then begin
       open_vdisplay_row "NOWRAP" ;
-      open_script_font () ;
-      scanner sup ;
+      clearstyle () ;
+      put sup ;
       close_vdisplay_row ()
     end ;           
     open_vdisplay_row "" ;
@@ -392,36 +369,36 @@ let standard_sup_sub scanner what sup sub display =
     close_vdisplay_row () ;
     if sub <> "" then begin
       open_vdisplay_row "NOWRAP" ;
-      open_script_font () ;
-      scanner sub ;
+      clearstyle () ;
+      put sub ;
       close_vdisplay_row ()
     end ;
-      close_vdisplay () ;
-      force_item_display ()
+    close_vdisplay () ;
+    force_item_display ()
   end else begin
-     what ();
-     put_sup_sub "SUB" scanner sub ;
-     put_sup_sub "SUP" scanner sup
+    what ();
+    reput_sup_sub "SUB" sub ;
+    reput_sup_sub "SUP" sup
   end
 ;;
 
 
 let limit_sup_sub scanner what sup sub display =
+  let sup = to_string (fun () -> put_sup_sub display scanner sup)
+  and sub = to_string (fun () -> put_sup_sub display scanner sub) in
   if sup = "" && sub = "" then
     what ()
   else begin
     force_item_display () ;
     open_vdisplay display ;
     open_vdisplay_row "ALIGN=center" ;
-    open_script_font () ;
-    scanner sup ;
+    put sup ;
     close_vdisplay_row () ;
-    open_vdisplay_row "ALIGN=center" ;
+    open_vdisplay_row "ALIGN=left" ;
     what () ;
     close_vdisplay_row () ;
     open_vdisplay_row "ALIGN=center" ;
-    open_script_font () ;
-    scanner sub ;
+    put sub ;
     close_vdisplay_row () ;
     close_vdisplay () ;
     force_item_display ()
@@ -429,6 +406,8 @@ let limit_sup_sub scanner what sup sub display =
 ;;
 
 let int_sup_sub something vsize scanner what sup sub display =
+  let sup = to_string (fun () -> put_sup_sub display scanner sup)
+  and sub = to_string (fun () -> put_sup_sub display scanner sub) in
   if something then begin
     force_item_display () ;
     what () ;
@@ -437,8 +416,7 @@ let int_sup_sub something vsize scanner what sup sub display =
   if sup <> "" || sub <> "" then begin
     open_vdisplay display ;
     open_vdisplay_row "ALIGN=left NOWRAP" ;
-    open_script_font () ;
-    scanner sup ;
+    put sup ;
     close_vdisplay_row () ;
     open_vdisplay_row "ALIGN=left" ;
     for i = 2 to vsize do
@@ -446,8 +424,7 @@ let int_sup_sub something vsize scanner what sup sub display =
     done ;
     close_vdisplay_row () ;
     open_vdisplay_row "ALIGN=left NOWRAP" ;
-    open_script_font () ;
-    scanner sub ;
+    put sub ;
     close_vdisplay_row () ;
     close_vdisplay () ;
     force_item_display ()
@@ -462,15 +439,15 @@ let insert_vdisplay open_fun =
   try
     let mods = to_pending !cur_out.pending !cur_out.active in
     let bs,bargs,bout = pop_out out_stack in
-    if bs <> "" then
-      failclose ("insert_vdisplay: "^bs^" closes ``''");
+    if bs <> INTERN then
+      failclose "insert_vdisplay" bs INTERN ;
     let ps,pargs,pout = pop_out out_stack in
-    if ps <> "TD" then
-      failclose ("insert_vdisplay: "^ps^" closes TD");
+    if ps <> TD then
+      failclose "insert_vdisplay" ps TD ;
     let pps,ppargs,ppout = pop_out out_stack  in
-    if pps <> "DISPLAY" then
-      failclose ("insert_vdisplay: "^pps^" closes DISPLAY");
-    let new_out = new_status false [] [] in
+    if pps <> DISPLAY then
+      failclose "insert_vdisplay" pps DISPLAY ;
+    let new_out = create_status_from_scratch false [] in
     push_out out_stack (pps,ppargs,new_out) ;
     push_out out_stack (ps,pargs,pout) ;
     push_out out_stack (bs,bargs,bout) ;    
@@ -482,14 +459,14 @@ let insert_vdisplay open_fun =
     free new_out ;    
     if !verbose > 2 then begin
       prerr_string "insert_vdisplay -> " ;
-      pretty_mods mods ;
+      pretty_mods stderr mods ;
       prerr_newline ()
     end ;
     if !verbose > 2 then
       prerr_flags "<= insert_vdisplay" ;
     mods
   with PopFreeze ->
-    raise (Error "\\over should be properly parenthesized")
+    raise (UserError "\\over should be properly parenthesized")
 ;;
 
 
@@ -536,9 +513,15 @@ let put_delim delim i =
   end
 ;;
 
-let left delim =
+let left delim k =
   let _,f,is_freeze = end_item_display () in
-  delay (fun vsize -> put_delim delim vsize) ;
+  delay
+    (fun vsize ->
+      put_delim delim vsize ;
+      begin_item_display (fun () -> ()) false ;
+      k vsize ;
+      let _ = end_item_display () in
+      ()) ;
   begin_item_display f is_freeze
 ;;
 
