@@ -10,13 +10,16 @@
 (***********************************************************************)
 
 exception Fatal of string
+exception NoSupport of string
 exception Purposly of string
 exception ScanError of string
+exception UserError of string
 exception EndInput
 exception EndDocument
 exception Close of string
 exception EndOfLispComment of int (* QNC *)
 
+val hot_start : unit -> unit
 val verbose : int ref
 val readverb : int ref
 val silent : bool ref
@@ -25,5 +28,20 @@ val warning : string -> unit
 val print_verb : int -> string -> unit
 val message : string -> unit
 val fatal : string -> 'a
+val not_supported : string -> 'a
+
+(* Copying hash tables, not very nice at present *)
 val copy_hashtbl : (string, 'a) Hashtbl.t -> (string, 'a) Hashtbl.t -> unit
 val clone_hashtbl : (string, 'a) Hashtbl.t -> (string, 'a) Hashtbl.t
+val copy_int_hashtbl : (int, 'a) Hashtbl.t -> (int, 'a) Hashtbl.t -> unit
+val clone_int_hashtbl : (int, 'a) Hashtbl.t -> (int, 'a) Hashtbl.t
+
+val start_env : string -> string
+val end_env : string -> string
+
+type limits = Limits | NoLimits | IntLimits
+
+val image_opt : string option ref
+val get_image_opt : unit -> string
+
+val dump_index : bool ref
