@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(*  $Id: package.ml,v 1.36 2002/04/29 14:31:03 maranget Exp $    *)
+(*  $Id: package.ml,v 1.43 2003/09/29 08:54:54 maranget Exp $    *)
 
 module type S = sig  end
 
@@ -49,7 +49,15 @@ def_print "\\@heveacomline"
 def_print "\\@heveaversion" Version.version ;
 def_print "\\@hevealibdir" Mylib.libdir
 ;;
+(* Unicode entities given in hexa *)
 
+def_code "\\@unicode"
+ (fun lexbuf ->
+   let arg = get_prim_arg lexbuf in
+   Scanf.sscanf arg "%x" (fun x -> Dest.put ("&#"^string_of_int x^";")))
+;;
+
+     
 (* ``Token'' registers *)
 def_code "\\newtokens"
   (fun lexbuf ->
@@ -116,12 +124,27 @@ and call_prim lexbuf =
     prerr_endline exec ;
   end ;
   scan_this  main exec
+
+and call_subst_opt lexbuf =
+  let csname = get_csname lexbuf in
+  let default = subst_arg lexbuf in
+  let arg = subst_arg lexbuf in
+  let lb = Lexing.from_string arg in
+  let opt = try Save.opt lb with Save.NoOpt -> default in
+  let rem = Save.remain lb in
+  let exec = csname ^ "{" ^ opt ^ "}"  ^ rem  in
+  if !verbose > 1 then begin
+    prerr_string "\\@callsubstopt: " ;
+    prerr_endline exec ;
+  end ;
+  scan_this  main exec
 ;;
 
 
 
 def_code "\\@funcall" call_subst ;
 def_code "\\@callsubst" call_subst ;
+def_code "\\@callsubstopt" call_subst_opt ;
 def_code "\\@callprim" call_prim ;
 ;;
 
@@ -172,7 +195,10 @@ def_code "\\@auxread"
 def_code "\\@bibread"
   (fun lexbuf ->
     let key = get_raw lexbuf in
-    scan_this main (Auxx.bget false key))
+    let arg = match Auxx.bget false key with
+    | None -> "\\@verbarg{"^key^"}"
+    | Some s -> s in
+    scan_this main arg)
 ;;
 
 def_code "\\@bibwrite"
@@ -383,6 +409,18 @@ fallback name");
     )
 ;;
 
+(* Some strange arg scanning, needed for \DeclareGraphicsRule *)
+
+register_init "graphics"
+  (fun () ->
+    def_code "\\@verbimagearg"
+      (fun lexbuf ->
+       let {arg=arg} = save_arg lexbuf in
+       Image.put_char '{' ;
+       Image.put arg ;
+       Image.put_char '}'))
+;;
+
 (* url package *)
 let verb_arg lexbuf =
   let {arg=url} = save_verbatim lexbuf in
@@ -427,6 +465,7 @@ register_init "url"
     ())
 ;;         
 
+
 (* hyperref (not implemented in fact) *)
 register_init "hyperref"
   (fun () ->
@@ -538,7 +577,11 @@ let do_setkey lexbuf =
         if value <> "" then
           scan_this main (csname^"{"^value^"}")
         else
-          scan_this main (csname^"@default")
+          let defname = csname^"@default" in
+          if Latexmacros.exists defname then
+            scan_this main defname
+          else
+            warning ("keyval, no default value for key: ``"^key^"''")
       end else
         warning ("keyval, uknown key: ``"^key^"''") ;
       do_rec ()
@@ -563,5 +606,38 @@ register_init "amsmath"
   )
 ;;
 
+register_init "xypic"
+  (fun () -> def_code "\\@xyarg" 
+      (fun lexbuf ->
+        Save.start_echo () ;
+        let _ = Lexstate.save_xy_arg lexbuf in
+        let r = Save.get_echo () in
+        Image.put r
+       )
+  )
+;;
 
+
+register_init "natbib"
+  (fun () ->
+    def_code "\\NAT@write"
+      (fun lexbuf ->
+        let key = get_raw lexbuf in
+        let num = get_prim_arg lexbuf in
+        let auth = subst_arg lexbuf in
+        let year = subst_arg lexbuf in
+        let long = subst_arg lexbuf in
+        Auxx.bwrite key
+          ("{"^num^"}{"^auth^"}{"^year^"}{"^long^"}")) ;
+    def_code "\\NAT@bibread"
+      (fun lexbuf ->
+        let key = get_raw lexbuf in
+        let arg =
+          match Auxx.bget false key with
+          | None -> "{}{??}{}{}"
+          | Some s -> s in
+        scan_this main ("\\NAT@args" ^ arg)) ;
+    ())
+            
+        
 end
