@@ -9,16 +9,19 @@
 (*                                                                     *)
 (***********************************************************************)
 
-let header = "$Id: text.ml,v 1.41 2000/01/26 19:39:50 maranget Exp $"
+let header = "$Id: text.ml,v 1.58 2003/02/13 14:51:01 maranget Exp $"
 
 
 open Misc
 open Parse_opts
+open Element
+open Lexstate
 open Latexmacros
 open Stack
 open Length
 
 exception Error of string;;
+type block = string
 
 
 let r_quote = String.create 1
@@ -101,12 +104,22 @@ let iso_translate = function
 ;;
 
 let iso c =
-  if !Parse_opts.iso then
+  if !Parse_opts.iso || !Lexstate.raw_chars then
     (r_translate.[0]<-c; r_translate)
   else
     iso_translate c
 ;;
 
+let iso_buff = Out.create_buff ()
+
+let iso_string s =
+  if !Parse_opts.iso then begin
+    for i = 0 to String.length s - 1 do
+      Out.put iso_buff (iso_translate s.[i])
+    done ;
+    Out.to_string iso_buff
+  end else
+    s
 
 
 let failclose s = raise (Misc.Close s)
@@ -116,7 +129,7 @@ let failclose s = raise (Misc.Close s)
 (* output globals *)
 type status = {
     mutable nostyle : bool ;
-    mutable active : env list ;
+    mutable active : text list ;
     mutable out : Out.t;
     mutable temp : bool
   };;
@@ -320,6 +333,7 @@ type stack_t = {
   s_in_table : bool Stack.t ;
   s_vsize : int Stack.t ;
   s_active : Out.t Stack.t ;
+  s_pending_par : int option Stack.t ;
   s_after : (string -> string) Stack.t
 } 
 
@@ -335,6 +349,7 @@ let stacks = {
   s_in_table = Stack.create "in_table" ;
   s_vsize = Stack.create "vsize" ;
   s_active = Stack.create "active" ;
+  s_pending_par = Stack.create "pending_par" ;
   s_after = Stack.create "after"
 } 
 
@@ -349,7 +364,8 @@ type saved_stacks = {
   ss_nocount : bool Stack.saved ;
   ss_in_table : bool Stack.saved ;
   ss_vsize : int Stack.saved ;
-  ss_active : Out.t Stack.saved ;
+  ss_active : Out.t Stack.saved ;  
+  ss_pending_par : int option Stack.saved ;
   ss_after : (string -> string) Stack.saved
 } 
 
@@ -366,6 +382,7 @@ let save_stacks () =
   ss_in_table = Stack.save stacks.s_in_table ;
   ss_vsize = Stack.save stacks.s_vsize ;
   ss_active = Stack.save stacks.s_active ;
+  ss_pending_par = Stack.save stacks.s_pending_par ;
   ss_after = Stack.save stacks.s_after
 }
 
@@ -382,6 +399,7 @@ and restore_stacks
   ss_in_table = saved_in_table ;
   ss_vsize = saved_vsize ;
   ss_active = saved_active ;
+  ss_pending_par = saved_pending_par ;
   ss_after = saved_after
 } =
   Stack.restore stacks.s_nitems saved_nitems ;
@@ -395,6 +413,7 @@ and restore_stacks
   Stack.restore stacks.s_in_table saved_in_table ;
   Stack.restore stacks.s_vsize saved_vsize ;
   Stack.restore stacks.s_active saved_active ;
+  Stack.restore stacks.s_pending_par saved_pending_par ;
   Stack.restore stacks.s_after saved_after
 
 let check_stack what =
@@ -417,6 +436,7 @@ let check_stacks () = match stacks with
   s_in_table = in_table ;
   s_vsize = vsize ;
   s_active = active ;
+  s_pending_par = pending_par ;
   s_after = after
 } ->
   check_stack nitems ;
@@ -430,6 +450,7 @@ let check_stacks () = match stacks with
   check_stack in_table ;
   check_stack vsize ;
   check_stack active ;
+  check_stack pending_par ;
   check_stack after
 
 let line = String.create (!Parse_opts.width +2);;
@@ -451,10 +472,12 @@ and hot (l,f,s,o) =
 
 let stop () =
   Stack.push stacks.s_active !cur_out.out ;
+  Stack.push stacks.s_pending_par flags.pending_par ;
   !cur_out.out <- Out.create_null ()
 
 and restart () =
-  !cur_out.out <- Stack.pop stacks.s_active
+  !cur_out.out <- Stack.pop stacks.s_active ;
+  flags.pending_par <- Stack.pop stacks.s_pending_par
 
 let do_do_put_char c =
   Out.put_char !cur_out.out c;;
@@ -483,7 +506,7 @@ let do_put_line s =
       let sp = flags.hsize - length + flags.x_start in
       String.concat "" [ String.make sp ' '; s]
   in
-  if !verbose>3 then prerr_endline ("line :"^ligne);
+  if !verbose > 3 then prerr_endline ("line :"^ligne);
   do_do_put ligne;
 
 
@@ -522,8 +545,8 @@ let do_flush () =
 ;;
   
 let do_put_char_format c =
-  if !verbose >3 then
-    prerr_endline ("caracters read :"^Char.escaped c^", x="^string_of_int flags.x^", length ="^string_of_int (flags.hsize));
+  if !verbose > 3 then
+    prerr_endline ("caracters read : '"^Char.escaped c^"', x="^string_of_int flags.x^", length ="^string_of_int (flags.hsize));
 
   if c=' ' then  flags.last_space <- flags.x;
   if flags.x =(-1) then begin
@@ -537,8 +560,8 @@ let do_put_char_format c =
   line.[flags.x]<-c;
   if c='\n' then begin
 	(* Ligne prete *)
-    if !verbose>2 then
-      prerr_endline("line not cut :"^line);
+    if !verbose > 2 then
+      prerr_endline("line not cut :["^line^"]");
     do_put_line (String.sub line 0 (flags.x +1));
     flags.x <- -1;
   end else
@@ -546,7 +569,7 @@ let do_put_char_format c =
   if flags.x>(flags.x_end +1) then begin (* depassement de ligne *)
     if (flags.x - flags.last_space) >= flags.hsize then begin
 	  (* On coupe brutalement le mot trop long *)
-      if !verbose >2 then
+      if !verbose > 2 then
 	prerr_endline ("line cut :"^line);
       warning ("line too long");
       line.[flags.x-1]<-'\n';
@@ -689,13 +712,13 @@ let par = function (*Nombre de lignes a sauter avant le prochain put*)
 	  (match pblock() with
 	  | "QUOTE" | "QUOTATION" -> Some (n-1)
 	  | _ -> Some n);
-	if !verbose>2 then
+	if !verbose> 2 then
 	  prerr_endline
 	    ("par: last_close="^flags.last_closed^
 	     " r="^string_of_int n);
       end
-  | _ -> flags.pending_par <- None
-;;
+  | _ -> ()
+
 
 let forget_par () = 
   let r = flags.pending_par in
@@ -706,7 +729,8 @@ let forget_par () =
 let flush_par n =
   flags.pending_par <- None;
   let p = n in
-  for i=1 to p do
+  do_put_char '\n' ;
+  for i=1 to p-1 do
     do_put_char '\n'
   done;
   if !verbose >2 then
@@ -741,6 +765,9 @@ let try_open_block s args =
     (flags.hsize,flags.x,flags.x_start,flags.x_end,
     flags.first_line,flags.last_space);
 
+  push stacks.s_align flags.align;
+  push stacks.s_in_align flags.in_align;
+
   if is_list s then begin
     do_put_char '\n';
     push stacks.s_nitems flags.nitems;
@@ -750,7 +777,6 @@ let try_open_block s args =
     flags.hsize <- flags.x_end - flags.x_start+1;
     
     if not flags.in_align then begin
-      push stacks.s_align flags.align;
       flags.align <- Left
     end;
     if s="DL" then begin
@@ -759,12 +785,10 @@ let try_open_block s args =
       flags.dt <- "";
       flags.dcount <- "";
     end;
-  end else match s with
+  end else begin match s with
   | "ALIGN" ->
       begin
 	finit_ligne ();	
-	push stacks.s_align flags.align;
-	push stacks.s_in_align flags.in_align;
 	flags.in_align<-true;
 	flags.first_line <-2;
 	match args with
@@ -783,8 +807,6 @@ let try_open_block s args =
   | "QUOTE" ->
       begin
 	finit_ligne ();
-	push stacks.s_align flags.align;
-	push stacks.s_in_align flags.in_align;
 	flags.in_align<-true;
 	flags.align <- Left;
 	flags.first_line<-0;
@@ -794,8 +816,6 @@ let try_open_block s args =
   | "QUOTATION" ->
       begin
 	finit_ligne ();
-	push stacks.s_align flags.align;
-	push stacks.s_in_align flags.in_align;
 	flags.in_align<-true;
 	flags.align <- Left;
 	flags.first_line<-2;
@@ -811,7 +831,13 @@ let try_open_block s args =
       push stacks.s_nocount flags.nocount ;
       flags.nocount <- true ;
       flags.first_line <-0
-  | _ -> ();
+  | "INFOLINE" ->
+      push stacks.s_nocount flags.nocount ;
+      flags.nocount <- true ;
+      flags.first_line <-0 ;
+      finit_ligne ()
+  | _ -> ()
+  end ;
 
   if !verbose > 2 then
     prerr_endline ("<= try_open ``"^s^"''")
@@ -824,40 +850,32 @@ let try_close_block s =
   flags.x_end<-xe;
   flags.first_line <-fl;
 
-
   if (is_list s) then begin
     finit_ligne();
-    if not flags.in_align then begin
-      let a = pop stacks.s_align in
-      flags.align <- a
-    end;
     flags.nitems <- pop  stacks.s_nitems;
     if s="DL" then begin
       flags.dt <- pop stacks.s_dt;
-      flags.dcount <- pop stacks.s_dcount;
-    end;
-  end else match s with
+      flags.dcount <- pop stacks.s_dcount
+    end
+  end else begin match s with
   | "ALIGN" | "QUOTE" | "QUOTATION" ->
-      begin
-	finit_ligne ();
-	let a = pop stacks.s_align in
-	flags.align <- a;
-	let ia = pop  stacks.s_in_align in
-	flags.in_align <- ia;
-      end
+	finit_ligne ()
   | "HEAD" ->
-      begin
-	finit_ligne();
-	let u = pop stacks.s_underline in
-	flags.underline <- u
-      end
+      finit_ligne();
+      let u = pop stacks.s_underline in
+      flags.underline <- u
   | "PRE" ->
       flags.first_line <-0;
       do_put ">>\n";
-      flags.first_line <-fl;
-  | "INFO" ->
+      flags.first_line <-fl
+  | "INFO"|"INFOLINE"->
       flags.nocount <- pop stacks.s_nocount
   | _ -> ()
+  end ;
+  let a = pop stacks.s_align in
+  flags.align <- a;
+  let ia = pop  stacks.s_in_align in
+  flags.in_align <- ia
 ;;
 
 let open_block s args =
@@ -904,9 +922,6 @@ let force_block s content =
   end else raise ( Misc.Fatal "text: unflushed DELAY")
 ;;
 
-let close_flow s = ()
-;;
-
 let close_block s =
   (* Fermeture du bloc : recuperation de la pile *)
   if !verbose > 2 then
@@ -919,14 +934,13 @@ let close_block s =
 
 
 
-let insert_block tag arg = 
-  if tag = "ALIGN" then begin
-    match arg with
-      "LEFT" -> flags.align <- Left
-    | "CENTER" -> flags.align <- Center
-    | "RIGHT" -> flags.align <- Right
-    | _ -> raise (Misc.ScanError "Invalid argument in ALIGN");
-  end;
+let insert_block tag arg =  match arg with
+| "LEFT" -> flags.align <- Left
+| "CENTER" -> flags.align <- Center
+| "RIGHT" -> flags.align <- Right
+| _ -> raise (Misc.ScanError "Invalid argument in ALIGN");
+
+and insert_attr _ _ = ()
 ;;
 
 
@@ -997,6 +1011,14 @@ let erase_block s =
   end
 ;;
 
+let to_string f =
+  open_block "TEMP" "";
+  f () ;
+  let r = Out.to_string !cur_out.out in
+  close_block "TEMP";
+  r
+;;
+
 let open_group ss =  
   open_block "" "";
   open_mod (Style ss);
@@ -1014,11 +1036,15 @@ let close_group () =
 
 
 let put s =
+  if !verbose > 3 then
+    Printf.fprintf stderr "put: %s\n" s ;
   do_pending ();
   do_put s
 ;;
 
 let put_char c =
+  if !verbose > 3 then
+    Printf.fprintf stderr "put_char: %c\n" c ;
   do_pending ();
   do_put_char c
 ;;
@@ -1028,6 +1054,8 @@ let flush_out () =
 ;;
 
 let skip_line () =
+  if !verbose > 2 then
+    prerr_endline "skip_line" ;
   put_char '\n'
 ;;
 
@@ -1044,13 +1072,6 @@ let close_chan () =
   !cur_out.out <- Out.create_buff()
 ;;
 
-let to_string f =
-  open_block "TEMP" "";
-  f () ;
-  let r = Out.to_string !cur_out.out in
-  close_block "TEMP";
-  r
-;;
 
 let to_style f =
   !cur_out.active<-[];
@@ -1105,7 +1126,7 @@ let put_in_math s =
 (*--------------*)
 
 type align = Top | Middle | Bottom | Base of int
-and wrap_t = True | False | Fill
+and wrap_t = Wtrue | Wfalse | Fill
 ;;
 
 
@@ -1146,12 +1167,28 @@ type table_t = {
   } 
 ;;
 
+let ptailles chan table =
+  let t = table.tailles in
+  Printf.fprintf chan  "[" ;
+  for i = 0 to Array.length t-1 do
+    Printf.fprintf chan "%d; " t.(i)
+  done ;
+  Printf.fprintf chan  "]"
+
+let ptaille chan table =
+  let t = Table.to_array table.taille in
+  Printf.fprintf chan  "[" ;
+  for i = 0 to Array.length t-1 do
+    Printf.fprintf chan "%d; " t.(i)
+  done ;
+  Printf.fprintf chan  "]"
+
 let cell = ref {
   ver = Middle;
   hor = Left;
   h = 0;
   w = 0;
-  wrap = False;
+  wrap = Wfalse;
   span = 1;
   text = "";
   pre  = "";
@@ -1226,7 +1263,7 @@ let open_table border _ =
     hor = Left;
     h = 0;
     w = 0;
-    wrap = False;
+    wrap = Wfalse;
     span = 1;
     text = "";
     pre  = "";
@@ -1239,19 +1276,41 @@ let open_table border _ =
   flags.in_table<-true;
 ;;
 
+let register_taille table =
+  let old = table.tailles
+  and cur = Table.trim table.taille in
+  let old_len = Array.length old
+  and cur_len = Array.length cur in
+  let dest = 
+    if cur_len > old_len then begin
+      let t = Array.create cur_len 0 in
+      Array.blit old 0 t 0 old_len ;
+      t
+    end else
+      old in
+  for i=0 to cur_len-1 do
+    dest.(i) <- max dest.(i) cur.(i)
+  done ;
+  table.tailles <- dest
+
 let new_row () =
+  if !verbose> 2 then begin
+    Printf.eprintf "=> new_row, line =%d, tailles=%a\n"
+      !table.line ptailles !table
+  end ;
   if !table.col> !table.cols then !table.cols<- !table.col;
   !table.col <- -1;
   !table.line <- !table.line +1;
-  if !table.line = 1 && (( Array.length !table.tailles)=0) then  !table.tailles<-Table.trim !table.taille;
-  let _=match !row.cells with
+  Table.reset !table.taille ;
+
+  let _ =match !row.cells with
   | Tabl t -> Table.reset t
   | _-> raise (Error "invalid table type in array")
   in
   !cell.pre <- "";
   !cell.pre_inside <- [];
   !row.haut<-0;
-  if !verbose>2 then prerr_endline ("new_row, line ="^string_of_int !table.line)
+  if !verbose>2 then prerr_endline ("<= new_row, line ="^string_of_int !table.line)
 ;;
 
 let change_format format = match format with 
@@ -1277,14 +1336,14 @@ let change_format format = match format with
       | "left" -> Left
       | "right" -> Right
       | _-> raise (Misc.Fatal ("open_cell, invalid horizontal format :"^h)));
-    !cell.wrap <- (if w then True else False);
+    !cell.wrap <- (if w then Wtrue else Wfalse);
     if w then
       !cell.w <- 
 	(match size with
 	| Length.Char l -> l
 	| Length.Pixel l -> l / Length.font
 	| Length.Percent l -> l * !Parse_opts.width / 100              
-	| Length.Default -> !cell.wrap <- False; warning "cannot wrap column with no width"; 0
+	| Length.Default -> !cell.wrap <- Wfalse; warning "cannot wrap column with no width"; 0
         | Length.No s ->
             raise (Misc.Fatal ("No-length ``"^s^"'' in out-manager")))
     else !cell.w <- 0;
@@ -1311,7 +1370,7 @@ let open_cell format span insides =
   !cell.post_inside <- [];
   open_block "" "";
   if !cell.w > String.length line then raise ( Error "Column too wide");
-  if (!cell.wrap=True) then begin (* preparation de l'alignement *)
+  if (!cell.wrap=Wtrue) then begin (* preparation de l'alignement *)
     !cur_out.temp <- false;
     flags.x_start <- 0;
     flags.x_end <- !cell.w-1;
@@ -1329,7 +1388,7 @@ let open_cell format span insides =
 
 let close_cell content =
   if !verbose>2 then prerr_endline "=> force_cell";
-  if (!cell.wrap=True) then begin
+  if (!cell.wrap=Wtrue) then begin
     do_flush ();
     flags.in_align <- pop stacks.s_in_align;
     flags.align <- pop stacks.s_align;
@@ -1342,13 +1401,13 @@ let close_cell content =
 				    "#,post :#"^ !cell.post^
 				    "#");
   (* il faut remplir les champs w et h de cell *)
-  if (!cell.wrap = False ) then !cell.w <- 0;
+  if (!cell.wrap = Wfalse ) then !cell.w <- 0;
   !cell.h <- 1;
   let taille = ref 0 in
   for i = 0 to (String.length !cell.text) -1 do
     if !cell.text.[i]='\n' then begin
       !cell.h<- !cell.h+1;
-      if (!cell.wrap = False) && (!taille > !cell.w) then begin
+      if (!cell.wrap = Wfalse) && (!taille > !cell.w) then begin
 	!cell.w <- !taille;
       end;
       taille:=0;
@@ -1356,7 +1415,7 @@ let close_cell content =
       taille:=!taille+1;
     end;
   done;
-  if (!cell.wrap = False) && (!taille > !cell.w) then !cell.w <- !taille;
+  if (!cell.wrap = Wfalse) && (!taille > !cell.w) then !cell.w <- !taille;
   !cell.w <- !cell.w + (String.length !cell.pre) + (String.length !cell.post);
   if !verbose>2 then prerr_endline ("size : width="^string_of_int !cell.w^
 				    ", height="^string_of_int !cell.h^
@@ -1371,7 +1430,7 @@ let close_cell content =
 		     span = !cell.span;
 		     text = !cell.text;
 		     pre  = !cell.pre;
-		     post = !cell.post;
+	     post = !cell.post;
 		     pre_inside  = !cell.pre_inside;
 		     post_inside = !cell.post_inside;
 		   }
@@ -1381,28 +1440,13 @@ let close_cell content =
   (* on a la taille de la cellule, on met sa largeur au bon endroit, si necessaire.. *)
   (* Multicolonne : Il faut mettre des zeros dans le tableau pour avoir la taille minimale des colonnes atomiques. Puis on range start,end dans une liste que l'on regardera a la fin pour ajuster les tailles selon la loi : la taille de la multicolonne doit etre <= la somme des tailles minimales. Sinon, il faut agrandir les colonnes atomiques pour que ca rentre. *)
   if !cell.span = 1 then begin
-    if !table.line = 0 then
-      Table.emit !table.taille !cell.w
-    else
-      begin
-	if !table.col >= (Array.length !table.tailles) then 
-	  begin (* depassement du tableau : on l'agrandit.. *)
-	    let t = Array.create (!table.col +1) 0 in
-	    Array.blit !table.tailles 0 t 0 (Array.length !table.tailles) ;
-	    !table.tailles <- t;
-	  end;
-	if (!cell.w > (!table.tailles.(!table.col))) then 
-	  begin
-	    !table.tailles.(!table.col)<- !cell.w;
-	  end;
-      end;
+    Table.emit !table.taille !cell.w
   end else if !cell.span = 0 then begin
-    if !table.line = 0 then Table.emit !table.taille 0;
+    Table.emit !table.taille 0;
   end else begin
-    if !table.line=0 then
-      for i = 1 to !cell.span do
-	Table.emit !table.taille 0
-      done;
+    for i = 1 to !cell.span do
+      Table.emit !table.taille 0
+    done;
     multi := (!table.col,!table.col + !cell.span -1,!cell.w) :: !multi;
   end;
   !table.col <- !table.col + !cell.span -1;
@@ -1425,7 +1469,7 @@ and erase_cell_group () = !table.in_cell <- false;
 
 let erase_cell () =
   if !verbose>2 then prerr_endline "erase cell";
-  if (!cell.wrap=True) then begin
+  if (!cell.wrap=Wtrue) then begin
     flags.in_align <- pop stacks.s_in_align;
     flags.align <- pop stacks.s_align;
   end;
@@ -1437,9 +1481,15 @@ let erase_cell () =
   !cell.pre_inside <- [];
 ;;
 
-let erase_row () = !table.line <- !table.line -1
+let erase_row () =
+  if !verbose > 2 then prerr_endline "erase_row" ;  
+  !table.line <- !table.line -1
+
 and close_row erase =
-  if !verbose>2 then prerr_endline "close_row";
+  if !verbose> 2  then
+    Printf.eprintf "close_row tailles=%a, taille=%a\n"
+      ptailles !table ptaille !table ;
+  register_taille !table ;
   Table.emit !table.table
      { haut = !row.haut;
      cells = Arr (Table.trim 
@@ -1522,8 +1572,9 @@ let put_ligne texte pos align width taille wrap=
   in
   let s = String.sub texte pos (pos_suiv - pos) in
   let t,post= 
-    if wrap=True then String.length s,0
+    if wrap=Wtrue then String.length s,0
     else width,width - String.length s in
+
   let ligne = match align with
   | Left -> String.concat "" 
 	[s; String.make (taille-t+post) ' ']
@@ -1549,11 +1600,12 @@ let put_border s inside j =
   done;
 ;;
 
-let rec somme debut fin = 
-  if debut = fin 
-  then !table.tailles.(debut)
-  else !table.tailles.(debut)
-      + (somme (debut+1) fin)
+let rec somme debut fin =
+  let r = ref 0 in
+  for k = debut to fin do
+    r := !r + !table.tailles.(k)
+  done ;
+  !r
 ;;
 
 
@@ -1594,7 +1646,9 @@ let close_table () =
     prerr_endline "=> close_table";
     pretty_stack out_stack
   end;
-  if !table.line=0 then  !table.tailles<-Table.trim !table.taille;
+
+  register_taille !table ;
+
   let tab = Table.trim !table.table in
   (* il reste a formatter et a flusher dans la sortie principale.. *)
   !table.lines<-Array.length tab;
@@ -1614,7 +1668,7 @@ let close_table () =
     in
     (* affichage de la ligne *)
     (* il faut envoyer ligne apres ligne dans chaque cellule, en tenant compte de l'alignement vertical et horizontal..*)
-    if !verbose>2 then prerr_endline ("line "^string_of_int i^", columns:"^string_of_int (Array.length ligne)^", height:"^string_of_int tab.(i).haut);
+    if !verbose> 2 then prerr_endline ("line "^string_of_int i^", columns:"^string_of_int (Array.length ligne)^", height:"^string_of_int tab.(i).haut);
     let pos = Array.create (Array.length ligne) 0 in
     !row.haut <-0;
     for j = 0 to tab.(i).haut -1 do
@@ -1622,11 +1676,11 @@ let close_table () =
       let col = ref 0 in
       for k = 0 to Array.length ligne -1 do
 	begin
-	  (* ligne j de la cellule k *)
+	  (* ligne j de la cellule k *)          
 	  if ligne.(k).wrap = Fill then ligne.(k).span <- Array.length !table.tailles;
 	  let taille_borders = (String.length ligne.(k).pre) + (String.length ligne.(k).post) in
 	  let taille = (somme !col (!col + ligne.(k).span-1)) - taille_borders in
-	  if !verbose>3 then prerr_endline ("cell to output:"^
+	  if !verbose> 2 then prerr_endline ("cell to output:"^
 					    ligne.(k).pre^
 					    ligne.(k).text^
 					    ligne.(k).post^
@@ -1788,7 +1842,7 @@ let pop_freeze () = match top  out_stack with
 ;;
 
 (* Displays *)
-let open_display args =
+let open_display _ =
   open_table (!verbose>1) "";
   new_row ();
   if !verbose > 1 then make_border "{";
@@ -1796,6 +1850,8 @@ let open_display args =
   open_cell_group ();
 ;;
 
+let open_display_varg _ = open_display ""
+
 let close_display () =
   if not (flush_freeze ()) then begin
     if !verbose > 1 then make_border "}";
@@ -1830,6 +1886,7 @@ let force_item_display () = item_display ()
 ;;
 
 let erase_display () = 
+  if !verbose > 2 then prerr_endline "erase_display" ;
   erase_cell ();
   erase_cell_group ();
   erase_row ();
@@ -1886,7 +1943,7 @@ and close_vdisplay_row () =
 let insert_sup_sub () =
   let f,is_freeze = pop_freeze () in
   let ps,parg,pout = pop_out out_stack in
-  if ps <> "" then failclose ("sup_sub : "^ps^"closes \"\"");
+  if ps <> "" then failclose ("sup_sub : "^ps^" closes \"\"");
   let new_out = newstatus false [] [] true in
   push_out out_stack (ps,parg,new_out);
   close_block "";
@@ -1904,7 +1961,7 @@ let insert_sup_sub () =
 let standard_sup_sub scanner what sup sub display =
   if display then begin
     insert_sup_sub ();
-    let f,ff = match sup,sub with
+    let f,ff = match sup.arg,sub.arg with
     | "","" -> "cm","cm"
     |	"",_ -> change_format (formated "lt"); "lb","cm"
     |	_,"" -> change_format (formated "lm"); "lt","cmm"
@@ -1912,7 +1969,7 @@ let standard_sup_sub scanner what sup sub display =
     in
     let vide= flags.empty in
     item_display_format f ;
-    if sup<>"" || sub<>"" then begin
+    if sup.arg <>"" || sub.arg<>"" then begin
       open_vdisplay display;
       (*if sup<>"" || vide then*) begin
 	open_vdisplay_row "lt";
@@ -1922,7 +1979,7 @@ let standard_sup_sub scanner what sup sub display =
       open_vdisplay_row "lm";
       what ();
       close_vdisplay_row ();
-      if sub<>"" || vide then begin
+      if sub.arg <>"" || vide then begin
 	open_vdisplay_row "lb";
 	scanner sub ;
 	close_vdisplay_row ();
@@ -1935,11 +1992,11 @@ let standard_sup_sub scanner what sup sub display =
     item_display ();
   end else begin
     what ();
-    if sub <> "" then begin
+    if sub.arg <> "" then begin
       put "_";
       scanner sub;
     end;
-    if sup <> "" then begin
+    if sup.arg <> "" then begin
       put "^";
       scanner sup;
     end;
@@ -2037,11 +2094,12 @@ let translate = function
 | s   -> s
 ;;
 
-let left delim =
+let left delim k =
   item_display ();
   open_display "";
   close_cell_group ();
   if delim<>"." then make_border (translate delim);
+  k 3 ;
   open_cell_group ();
 ;;
 
@@ -2053,3 +2111,7 @@ let right delim =
   vsize
 ;;
 
+(*
+  C'est fini, legamment
+*)
+
